@@ -62,7 +62,7 @@ public class GeneratorState extends RubyObject {
      * <code>0</code> means disabled.
      */
     private int maxNesting = DEFAULT_MAX_NESTING;
-    static final int DEFAULT_MAX_NESTING = 19;
+    static final int DEFAULT_MAX_NESTING = 100;
     /**
      * Whether special float values (<code>NaN</code>, <code>Infinity</code>,
      * <code>-Infinity</code>) are accepted.
@@ -262,6 +262,20 @@ public class GeneratorState extends RubyObject {
         String name = vName.asJavaString();
         if (getMetaClass().isMethodBound(name, true)) {
             return send(context, vName, Block.NULL_BLOCK);
+        } else {
+            IRubyObject value = getInstanceVariables().getInstanceVariable("@" + name);
+            return value == null ? context.nil : value;
+        }
+    }
+
+    @JRubyMethod(name="[]=", required=2)
+    public IRubyObject op_aset(ThreadContext context, IRubyObject vName, IRubyObject value) {
+        String name = vName.asJavaString();
+        String nameWriter = name + "=";
+        if (getMetaClass().isMethodBound(nameWriter, true)) {
+            return send(context, context.getRuntime().newString(nameWriter), value, Block.NULL_BLOCK);
+        } else {
+            getInstanceVariables().setInstanceVariable("@" + name, value);
         }
         return context.getRuntime().getNil();
     }
@@ -445,7 +459,7 @@ public class GeneratorState extends RubyObject {
      * @param vOpts The options hash
      * @return The receiver
      */
-    @JRubyMethod
+  @JRubyMethod(alias = "merge")
     public IRubyObject configure(ThreadContext context, IRubyObject vOpts) {
         OptionsReader opts = new OptionsReader(context, vOpts);
 
@@ -480,9 +494,9 @@ public class GeneratorState extends RubyObject {
      *
      * <p>Returns the configuration instance variables as a hash, that can be
      * passed to the configure method.
-     * @return
+     * @return the hash
      */
-    @JRubyMethod
+    @JRubyMethod(alias = "to_hash")
     public RubyHash to_h(ThreadContext context) {
         Ruby runtime = context.getRuntime();
         RubyHash result = RubyHash.newHash(runtime);
@@ -498,6 +512,9 @@ public class GeneratorState extends RubyObject {
         result.op_aset(context, runtime.newSymbol("max_nesting"), max_nesting_get(context));
         result.op_aset(context, runtime.newSymbol("depth"), depth_get(context));
         result.op_aset(context, runtime.newSymbol("buffer_initial_length"), buffer_initial_length_get(context));
+        for (String name: getInstanceVariableNameList()) {
+            result.op_aset(context, runtime.newSymbol(name.substring(1)), getInstanceVariables().getInstanceVariable(name));
+        }
         return result;
     }
 
